package workshop.core;

import workshop.utils.ConfRegister;

import java.lang.Math;
import java.util.*;

/**
 * @author Michele
 * @author Beniamino
 * @version 1.00
 *
 * Classe Job. Implementa un lavoro da eseguire e mantiene la lista degli
 * strumenti di cui ha bisogno, il tempo necessario per l'esecuzione del lavoro,
 * e quante volte il lavoro  stato "visitato" ma non assegnato.
 */

public class Job
{
   /**
    * Il costruttore. Crea un nuovo job con il relativo jobid.
    */

   public Job(long jobid)
   {
      long delta = ConfRegister.MAXTIME_JOB_NEED - ConfRegister.MINTIME_JOB_NEED;
      long jobTimeNeeds;
      
      String toolkey;
      
      id = jobid;
      
      /* Genera il tempo necessario all'esecuzione del lavoro */

      jobTimeNeeds = (long) ( (Math.random() * delta) + ConfRegister.MINTIME_JOB_NEED );
      timeNeeded = jobTimeNeeds;
      
      /* Genera la lista degli strumenti necessari per l'esecuzione del job */

      for (Enumeration e = Toolbox.tools().getToolNames() ; e.hasMoreElements() ;)
      {
         toolkey = e.nextElement().toString();
         int max = Toolbox.tools().selectTool(toolkey).getMaxCopies();
         int copiesNeeded = (int) Math.round( (Math.random() * (max - 1)) + 1);  // between 1 and max
         
         /* Qui controlliamo la probabilit che il tool corrente sia necessario,
          * per simulare la maggiore probabilit che uno strumento possa non
          * essere necessario. */

         if (Math.random() > ConfRegister.ZERO_TOOL_NEEDED_PROBABILITY)
         {
            tools.add( new Tool(toolkey, copiesNeeded) );
         }
      } 
   }
   
   /**
    * @return Il tempo necessario per eseguire il job
    */

   public long getTimeNeeded()
   {
      return timeNeeded;
   }

   /**
    * @return Gli strumenti necessari per eseguire il job
    */

   public List<Tool> getToolsNeeded()
   {
      return tools;
   }

   /**
    * Incrementa il contatore del numero di visite al job, senza che il job sia stato
    * assegnato ad un operaio.
    */

   public void increaseTimesSkipped()
   {
      this.timesSkipped++; 
   }

   /**
    * @return Il numero di visite al job senza averlo assegnato ad un operaio
    */

   public int getTimesSkipped ()
   {
      return (this.timesSkipped); 
   }
   
   /**
    * @return L'identificativo dell'instanza del job
    */

   public long getID()
   {
      return id;
   }
   
   private long   id;
   private long   timeNeeded;
   private int    timesSkipped = 0;
   private List<Tool> tools = new ArrayList<Tool>();
}