package workshop.core;

import java.lang.Math;
import java.util.List;

import workshop.utils.ConfRegister;

/**
 * @author Michele
 * @author Beniamino
 * @version 1.00
 *
 * Classe JobsGenerator. Implementa un thread che genera i lavori da far
 * svolgere agli operai e li inserisce nella coda dei lavori in attesa.
 */

public class JobsGenerator implements Runnable
{
   /**
    * Il costruttore
    */

   private JobsGenerator()
   {
      selfThread = new Thread(this, "JobsGenerator");
      selfThread.start();
   }
   
   /**
    * Il metodo RUN per il thread
    */

   public void run()
   {
      final int requested = ConfRegister.getJobRequested();

      while (true)
      {
         if (status == PAUSE)
         {
            try
            {
               synchronized (this)
               {
                  this.wait();
               }
            }
            catch (InterruptedException ie)
            {
            }
         }

         /* Quando raggiungiamo il limite di job richiesti, fermiamo il thread */

         if (jobid >= requested)
            break;

         /* Creiamo un nuovo job e aggiorniamo il counter */
         jobid++;
         Job newJob = new Job(jobid);
         
         Logger.debug("Creato il nuovo job n. " + jobid + "\n", this.selfThread);
         
         /* Inseriamo il job nella coda dei lavori in attesa */

         JobsSupervisor.self().addJob(newJob);
         
         /* Facciamo un po' di pausa per simulare l'arrivo dei lavori a tempi
          * casuali. I clienti non arrivano tutti in fila... */

         try
         {
            long delta = ConfRegister.MAXTIME_BETWEEN_JOB_GENERATION - ConfRegister.MINTIME_BETWEEN_JOB_GENERATION;
            long sleepingTime = (long) Math.round( (Math.random() * delta) + ConfRegister.MINTIME_BETWEEN_JOB_GENERATION );
            
            this.selfThread.sleep(sleepingTime);
         }
         catch (InterruptedException ie)
         {
            Logger.warning("Interrupt received by JobsGenerator");
         }
      }
   }
   
   /**
    * Accessor per l'unica istanza della classe
    *
    * @return L'istanza della classe JobsGenerator corrente
    */

   public static JobsGenerator self()
   {
      return DEFAULT_GENERATOR;
   }

   /**
    * @return Il numero di job generati
    */

   public static int getJobsGenerated()
   {
      return jobid;
   }

   private static int    jobid = 0;

   private static JobsGenerator DEFAULT_GENERATOR = new JobsGenerator();
   
   private Thread selfThread = null;
   
   private volatile int status = RUN;
   
   public static final int RUN    = 0;
   public static final int PAUSE  = 1;
   
   public synchronized void pause()
   {
      if (status == RUN)
         status = PAUSE;
   }
   
   public synchronized void resume()
   {
      if (status == PAUSE)
      {
         status = RUN;
         this.notify();
      }
   }
}