package workshop.core;

import workshop.core.exceptions.*;
import workshop.utils.*;

public class Laborer implements Runnable
{
   public Laborer(String name)
   {
      selfThread = new Thread(this, name);
      selfThread.start();
   }
   
   public void assignJob(Job theJob)
   {
      jobInProgress = theJob;
      
      synchronized (this)
      {
         this.notify();
      }
   }
   
   public void releaseTools()
   {
      for (Tool t : jobInProgress.getToolsNeeded() )
      {
         try
         {
            Toolbox.tools().selectTool(t.getName()).increase(t.getCopies());
         }
         catch (OverflowException oe)
         {
            Logger.error("Errore nel rilascio delle risorse", oe);
         }
         catch (IllegalArgumentException iae)
         {
            Logger.error("Errore nel rilascio delle risorse", iae);
         }
      }
   }
   
   public static int getAllJobsDone()
   {
      return allJobsDone;
   }

   public void run()
   {
      long jobExecutionStart, jobExecutionCurrent;
      long waitingJobStart, waitingJobCurrent;
      
      while (true)
      {
         if (status == PAUSE)
         {
            try
            {
               synchronized (this)
               {
                  this.wait();
               }
            }
            catch (InterruptedException ie)
            {
            }
         }

         /* Attendo un job */
         
         status = W_JOB;
         
         waitingJobStart = System.currentTimeMillis();
         
         try
         {
            synchronized (this)
            {
               if (jobInProgress == null)
               {
                  Logger.debug("Attendo un job da eseguire!\n", this.selfThread, false);
                  this.wait();
                  Logger.debug("Ho ottenuto un job da eseguire!\n", this.selfThread, false);
               }
            }
         }
         catch (InterruptedException ie)
         {
         }
         
         waitingJobCurrent = System.currentTimeMillis() - waitingJobStart;
         
         synchronized (Statistics.lock)
         {
            waitingJobAverage = (waitingJobAverage * allJobsDone + waitingJobCurrent) / (allJobsDone + 1);
   
            if (waitingJobCurrent > waitingJobMax)
               waitingJobMax = waitingJobCurrent;
         }
         
         status = RUN;
         
         jobExecutionStart = System.currentTimeMillis();

         /* Eseguo il job */
         
         synchronized (Logger.self())
         {
            Logger.log("Mi e' stato assegnato il job " + String.valueOf(jobInProgress.getID()) +
                       " e servono gli strumenti: \n", this.selfThread);
         
            for (Tool t : jobInProgress.getToolsNeeded() )
            {
               Logger.log("\t  " + t.getName() + " -> " + String.valueOf(t.getCopies()) + "\n", this.selfThread);
            }
         }

         try
         {
            this.selfThread.sleep(jobInProgress.getTimeNeeded());

            myJobsDone += 1;
         }
         catch (InterruptedException ie)
         {
            Logger.error("\n" + this.selfThread.getName() + "\n", ie);
         }

         releaseTools();
         
         jobInProgress = null;
         JobsSupervisor.self().addLaborer(this);

         Logger.debug("Job terminato!\n", this.selfThread, false);
         
         jobExecutionCurrent = System.currentTimeMillis() - jobExecutionStart;
         
         synchronized (Statistics.lock)
         {
            jobExecutionAverage = (jobExecutionAverage * allJobsDone + jobExecutionCurrent) / (allJobsDone + 1);
   
            if (jobExecutionCurrent > jobExecutionMax)
               jobExecutionMax = jobExecutionCurrent;
   
            allJobsDone += 1;
         }
      }
   }
   
   /**
    * @return Il tempo medio di esecuzione di un lavoro
    */

   public static synchronized long getJobExecutionAverageTime()
   {
      return jobExecutionAverage;
   }

   /**
    * @return Il tempo massimo con cui  stato eseguito un lavoro
    */
   
   public static synchronized long getJobExecutionMaxTime()
   {
      return jobExecutionMax;
   }

   /**
    * @return Il tempo medio di attesa per un nuovo lavoro
    */

   public static synchronized long getWaitingJobAverageTime()
   {
      return waitingJobAverage;
   }

   /**
    * @return Il tempo massimo che un operaio ha atteso prima di ottenere un
    *         nuovo lavoro
    */
   
   public static synchronized long getWaitingJobMaxTime()
   {
      return waitingJobMax;
   }
   
   private Job          jobInProgress  = null;
   
   private int          myJobsDone     = 0;
   private static volatile int   allJobsDone    = 0;
   
   private static volatile long jobExecutionAverage = 0;
   private static volatile long jobExecutionMax = 0;
   
   private static volatile long waitingJobAverage = 0;
   private static volatile long waitingJobMax = 0;

   private Thread selfThread = null;
   
   private volatile int status = RUN;
   
   public static final int RUN    = 0;
   public static final int PAUSE  = 1;
   public static final int W_JOB  = 2;
   
   public synchronized void pause()
   {
      if (status == RUN)
         status = PAUSE;
   }
   
   public synchronized void resume()
   {
      if (status == PAUSE)
      {
         status = RUN;
         this.notify();
      }
   }
}