package workshop.utils;

import workshop.core.Logger;

public class CommandLineParser
{
   private String[] commandLineArgs;

   public CommandLineParser(String[] args)
   {
      commandLineArgs = args;
      
      if (!controlInput())
         Logger.error("");
   }
   
   private final int DEFAULT_LABORER_NUMBER  = 6;
   private final int MAX_LABORER_NUMBER      = 100;
   private final int MIN_LABORER_NUMBER      = 4;
   private final int DEFAULT_JOB_NUMBER      = 1000;   
   private final int MAX_JOB_NUMBER          = 5000;
   private final int MIN_JOB_NUMBER          = 1000;
   
   //help's string 
   private final String HELP = new String(
      "\n"+
      "workshop -i filepath [-o filepath] [-l laborernumber][-j jobnunmber]\n"+ 
      "\n"+
      "-i filepath         Specifica la path del file di input\n"+
      "-l laborernumber    Specifica il numero di operai (Default 4)\n"+
      "-j jobnunmber       Specifica il numero di job (Default 1000)\n"+

      "Formato del file di input\n"       +
      "  nome_strumento numero_copie\n"   +
      "  nome_strumento numero_copie\n"   +
      "  ...... ..............\n"         +
      "  nome_strumento numero_copie\n"
   );

   private final char PARAMETER_SYMBOL ='-'; 
   
   //input filepath     
   private final char INPUT_FILE_PATH_PAR='i'; 

   //laborernumber       
   private final char  LAB_NUM_PAR='l';

   //jobnunmber   
   private final char  JOB_NUM_PAR='j';

   private String inputFilePath;
   private int    laborerNumber  = DEFAULT_LABORER_NUMBER;
   private int    jobNumber      = DEFAULT_JOB_NUMBER;

   public String getInputFilePath()
   {
      return inputFilePath;
   }
   
   public int getLaborerNumber()
   {
      return laborerNumber;
   }
   
   public int getJobNumber()
   {
      return jobNumber;
   }

   private boolean controlInput()
   {
      boolean isControlOk = true;
      int i;

      for(i = 0; i < commandLineArgs.length; i += 2)
      {
         if (commandLineArgs[i].toCharArray()[0] == PARAMETER_SYMBOL)
         {
            switch (commandLineArgs[i].toCharArray()[1])
            {
               case INPUT_FILE_PATH_PAR:
                  inputFilePath = new String(commandLineArgs[i+1]);
                  break;
         
               case LAB_NUM_PAR:
                  try
                  {
                     laborerNumber = Integer.parseInt(commandLineArgs[i+1]);

                     if (laborerNumber < MIN_LABORER_NUMBER || laborerNumber > MAX_LABORER_NUMBER)
                     {
                        Logger.warning("E' richiesto un numero di operai compreso tra " +
                                      String.valueOf(MIN_LABORER_NUMBER) + " e " +
                                      String.valueOf(MAX_LABORER_NUMBER)
                                    );
                        isControlOk = false;
                     }
                  }       
                  catch(Exception e)
                  {
                     isControlOk = false;
                  }

                  break;

               case JOB_NUM_PAR:
                  try
                  {
                     jobNumber = Integer.parseInt(commandLineArgs[i+1]);
                              
                     if (jobNumber < MIN_JOB_NUMBER || laborerNumber>this.MIN_JOB_NUMBER)
                     {
                        Logger.warning("E' richiesto un numero di lavori da eseguire compreso tra " +
                                      String.valueOf(MIN_JOB_NUMBER) + " e " +
                                      String.valueOf(MAX_JOB_NUMBER)
                                    );
                        isControlOk = false;
                     }
                  }       
                  catch(Exception e)
                  {
                     isControlOk = false;
                  }

                  break;

               default:
                  isControlOk = false;
                  break;
            }
         }
         else
         {
            isControlOk = false;
         }
         
         if (!isControlOk)
            break;
      }
      
      if (!isControlOk || inputFilePath == null)
      {
         System.out.println(HELP);
      }

      return (isControlOk && (inputFilePath != null));
   }
}
