package workshop.core;

/**
 * @author Michele
 * @author Beniamino
 * @version 0.1
 *
 * Logger class. It's not derivable, so it's declared as final.
 */

public final class Logger
{
   private Logger()
   {
   }
   
   /**
    * Stampa una stringa di debug, se la modalit di Debug  attivo
    *
    * @param message Il messaggio
    */
   
   public static void debug(String message)
   {
      if (DEBUG)
         System.out.print(message);
   }
   
   /**
    * Stampa una stringa di debug, se la modalit di Debug  attivo
    *
    * @param message Il messaggio
    * @param active  Falso se la stringa di debug va ignorata
    */

   public static void debug(String message, boolean active)
   {
      if (active)
         debug(message);
   }

   /**
    * Stampa una stringa di debug, se la modalit di Debug  attivo
    *
    * @param message Il messaggio
    * @param o       Il thread (solitamente this) che indica l'originatore del messaggio
    */

   public static void debug(String message, Thread o)
   {
      debug(o.getName() + " : " + message);
   }

   /**
    * Stampa una stringa di debug, se la modalit di Debug  attivo
    *
    * @param message Il messaggio
    * @param o       Il thread (solitamente this) che indica l'originatore del messaggio
    * @param active  Falso se la stringa di debug va ignorata
    */

   public static void debug(String message, Thread o, boolean active)
   {
      if (active)
         debug(o.getName() + " : " + message);
   }
   
   /**
    * Stampa un messaggio e forza l'uscita dal programma
    *
    * @param error Il messaggio d'errore da stampare
    */
   
   public static void error(String error)
   {
      System.err.print("Fatal Error: " + error);
      System.exit(1);
   }
   
   /**
    * Stampa un messaggio, lo stack trace dell'errore, e forza l'uscita dal programma
    *
    * @param error Il messaggio d'errore da stampare
    */
   
   public static void error(String error, Exception e)
   {
      System.err.print("Fatal Error: " + error + "\n\n");
      e.printStackTrace(System.err);
      System.exit(1);
   }
   
   /**
    * Registra un messaggio sullo standard output
    *
    * @param message Il messaggio
    */

   public static void log(String message)
   {
      System.out.print("Log: " + message);
   }

   /**
    * Registra un messaggio sullo standard output
    *
    * @param message Il messaggio
    * @param o       Il thread (solitamente this) che indica l'originatore del messaggio
    */

   public static void log(String message, Thread o)
   {
      System.out.print(o.getName() + ": " + message);
   }

   /**
    * Stampa una semplice stringa di warning
    *
    * @param warn Il messaggio di warning
    */
   
   public static void warning(String warn)
   {
      System.err.print("Warning: " + warn);
   }
   
   public static Logger self()
   {
      return DEFAULT_LOGGER;
   }
      
   public static final boolean DEBUG = true;
   
   private static Logger DEFAULT_LOGGER = new Logger();
}